from PySide6.QtQml import QQmlApplicationEngine
from PySide6.QtCore import QObject, Signal

from artemis.utils.path_utils import *
from artemis.utils.generic_utils import *

from artemis.utils.constants import Constants


class UIspaceweather(QObject):
    # Python > QML Signals
    show_ui = Signal()
    load_poseidon_report = Signal(dict)
    load_poseidon_forecast_report = Signal(dict)
    load_poseidon_drap_report = Signal(dict)
    load_poseidon_SSA_report = Signal(dict)
    load_poseidon_sun_images_report = Signal(dict)
    load_aurora_report = Signal()
    update_bottom_bar = Signal(str)


    def __init__(self, parent):
        super().__init__()

        self._parent = parent

        self._engine = QQmlApplicationEngine()
        self._engine.load('qrc:/ui/SpaceWeather.qml')
        self._window = self._engine.rootObjects()[0]

        self._window_current = self._window.findChild(QObject, "spaceWeatherCurrentObj")
        self._window_forecast = self._window.findChild(QObject, "spaceWeatherForecastObj")
        self._window_drap = self._window.findChild(QObject, "spaceWeatherDRAPObj")
        self._window_aurora = self._window.findChild(QObject, "spaceWeatherAuroraObj")
        self._window_SSA = self._window.findChild(QObject, "spaceWeatherSSA")
        self._window_sun_images = self._window.findChild(QObject, "spaceWeatherSunImagers")

        self._connect()


    def _connect(self):
        # QML > Python connections

        # Python > QML connections
        self.show_ui.connect(self._window.show)
        self.update_bottom_bar.connect(self._window.updateBottomBar)
        self.load_poseidon_report.connect(self._window_current.loadReport)
        self.load_poseidon_forecast_report.connect(self._window_forecast.loadForecastReport)
        self.load_poseidon_drap_report.connect(self._window_drap.loadDrapReport)
        self.load_poseidon_SSA_report.connect(self._window_SSA.loadDrapReport)
        self.load_poseidon_sun_images_report.connect(self._window_sun_images.loadDrapReport)
        self.load_aurora_report.connect(self._window_aurora.loadAuroraReport)


    def load_spaceweather_ui(self):
        """ Before opening the windows, poseidon report (data.json) is read online
        """
        self.download_poseidon_report()
    

    def download_poseidon_report(self):
        update_manager = self._parent.update_manager
        poseidon_data = update_manager.fetch_remote_json(Constants.POSEIDON_REPORT_URL)
        
        if poseidon_data:
            self.load_poseidon_report.emit(poseidon_data)
            self.load_poseidon_forecast_report.emit(poseidon_data)
            self.load_poseidon_drap_report.emit(poseidon_data)
            self.load_poseidon_sun_images_report.emit(poseidon_data)
            self.load_poseidon_SSA_report.emit(poseidon_data)
            self.load_aurora_report.emit()

            self.update_bottom_bar.emit(
                'Loaded Poseidon report issued on {} at {} UTC'.format(
                    poseidon_data['JSON_INFO']['utc_date'],
                    poseidon_data['JSON_INFO']['utc_time']
                )
            )
            self.show_ui.emit()



import os

from PySide6.QtQml import QQmlApplicationEngine
from PySide6.QtCore import QObject, Signal, Slot

from artemis.utils.path_utils import normalize_dialog_path
from artemis.utils.sys_utils import copy_file, open_file, delete_file
from artemis.utils.constants import Messages


class UIdocumentsmanager(QObject):
    # Python > QML Signals
    show_ui = Signal()
    close_ui = Signal()
    populate_documents_list = Signal(list)


    def __init__(self, parent):
        super().__init__()

        self._parent = parent

        self._engine = QQmlApplicationEngine()
        self._engine.load('qrc:/ui/DocumentsManager.qml')
        self._window = self._engine.rootObjects()[0]

        self._connect()


    def _connect(self):
        # QML > Python connections
        self._window.saveNewDoc.connect(self.save_new_doc)
        self._window.deleteDoc.connect(self.delete_doc)
        self._window.updateDoc.connect(self.update_doc)
        self._window.openDoc.connect(self.open_doc)


        # Python > QML connections
        self.show_ui.connect(self._window.show)
        self.close_ui.connect(self._window.close)
        self.populate_documents_list.connect(self._window.loadList)


    def load_documentsmanager_ui(self):
        self.load_documents_list()
        self.show_ui.emit()


    def load_documents_list(self):
        """ Load the documents of the selected signal and populate the documents list
        """
        self._parent.loaded_sig.select_documents()
        all_documents = self._parent.loaded_sig.documents
        
        keys = (
            'doc_id',
            'extension',
            'name',
            'description',
            'type',
            'preview'
        )
        
        doc_lst = [dict(zip(keys, values)) for values in all_documents]
        self.populate_documents_list.emit(doc_lst)


    @Slot(list)
    def save_new_doc(self, doc_lst):
        """ Save the new document (identified by the DOC_ID = -1) and reload the document list.
            doc_param contains all the details of the new documents. 
        """
        doc_param = doc_lst.toVariant()
        filename, file_extension = os.path.splitext(doc_param[0])

        doc_id = self._parent.loaded_sig.insert_document([
            -1,
            file_extension[1:],
            doc_param[1],
            doc_param[2],
            doc_param[3],
            0
        ])

        local_file_name = f'{doc_id}{file_extension}'
        origin_path = normalize_dialog_path(doc_param[0])
        copy_file(origin_path, self._parent.loaded_db.media_dir / local_file_name)
        self.load_documents_list()


    @Slot(list)
    def update_doc(self, doc_lst):
        """ Update the details of the existent document
        """
        doc_list = doc_lst.toVariant()
        for doc in doc_list:
            self._parent.loaded_sig.update_documents(doc[0], doc[1], doc[2], doc[3], doc[4])
        self._parent.load_sig(self._parent.loaded_sig.sig_id)
        self.load_documents_list()


    @Slot(str, str)
    def open_doc(self, doc_id, extension):
        """ Open the selected document with the proper system application (if any)
        """
        try:
            open_file(self._parent.loaded_db.media_dir / f'{doc_id}.{extension}')
        except Exception as e:
            self.close_ui.emit()
            self._parent.dialog_popup(
                Messages.DIALOG_TYPE_ERROR,
                Messages.GENERIC_ERROR,
                str(e)
            )


    @Slot(str, str, str, bool)
    def delete_doc(self, doc_id, doc_extension, doc_type, doc_preview):
        """ Delete the selected document
        """
        doc_file_name = f'{doc_id}.{doc_extension}'
        doc_file_path = self._parent.loaded_db.media_dir / doc_file_name
        
        self._parent.loaded_sig.delete_document(doc_id)
        
        if doc_preview:
            if doc_type == 'Audio':
                self._parent.lock_audio_player.emit()

        delete_file(doc_file_path)
        self.load_documents_list()



import locale
import sys

from PySide6.QtCore import qVersion


class Constants():
    """ Container class for several constants of the software """

    APPLICATION_NAME            = 'Artemis'
    ORGANIZATION_NAME           = 'AresValley'
    ORGANIZATION_DOMAIN         = 'aresvalley.com'
    APPLICATION_VERSION         = '4.2.0'

    SQL_NAME                    = 'data.sqlite'

    LATEST_VERSION_URL          = 'https://raw.githubusercontent.com/AresValley/Artemis/master/config/release-info.json'
    POSEIDON_REPORT_URL         = 'https://www.aresvalley.com/poseidon_engine/data.json'

    DEFAULT_ENCODING            = 'utf-8'
    SYSTEM_LANGUAGE             = 'en_US' # locale.getdefaultlocale()[0]
    PYTHON_VERSION              = '.'.join(str(v) for v in sys.version_info[:3])
    QT_VERSION                  = qVersion()


class Messages:
    """ Container class for messages to be displayed """
    # Type
    DIALOG_TYPE_INFO            = 'info'
    DIALOG_TYPE_QUEST           = 'question'
    DIALOG_TYPE_WARN            = 'warn'
    DIALOG_TYPE_ERROR           = 'error'

    # Titles
    GENERIC_SUCCESS             = "Success!"
    GENERIC_ERROR               = "Something went wrong!"
    NO_DB_DETECTED              = "No SigID database detected..."
    NO_CONNECTION               = "Connection Error!"
    UP_TO_DATE                  = "You're up to date!"
    DB_NEW_VER                  = "New SigID DB version available!"
    ART_NEW_VER                 = "New Artemis version available!"
    DB_CORRUPTED                = "Database Corruption Detected"

    # Messages
    DB_CREATION_SUCCESS_MSG     = "The new database has been created succesfully."
    GENERIC_ERROR_MSG           = "An error occurred during the process. Details: {}"
    IMPORTING_SUCCESS_MSG       = "Database importing has been succesfully completed!"
    EXPORTING_SUCCESS_MSG       = "Database exporting has been succesfully completed!"
    FILE_NOT_FOUND_ERR_MSG      = "The file you are trying to access cannot be located. This may be because the file has been moved or deleted."
    NO_DB_DETECTED_MSG          = "Do you want to download it now?"
    NO_CONNECTION_MSG           = "Unable to check for updates. It appears that there is a problem with your internet connection. Please check your network settings and try again later. {}"
    UP_TO_DATE_MSG              = "The latest version of Artemis and SigID wiki is installed on your computer."
    DB_NEW_VER_MSG              = "A new version of the database ({}) is available for download. Download now?"
    ART_NEW_VER_MANUAL_MSG      = "A new version of Artemis ({}) is available for download. Check GitHub page now?"
    ART_NEW_VER_AUTO_MSG        = "A new version of Artemis ({}) is available for download. Update Artemis now?"
    DB_CORRUPTED_MSG            = "Downloaded data corrupted or invalid. Please retry."
    DB_DOWNLOAD_SUCCESS_MSG     = "The database has been successfully downloaded and is now being loaded."



import os
from pathlib import Path

from artemis.utils.constants import Constants
from artemis.utils.sys_utils import is_windows, is_linux, is_macos


def normalize_dialog_path(path):
    if is_windows():
        norm_path = path.replace('file:///', '')
    elif is_linux() or is_macos():
        norm_path = path.replace('file:///', '/')
    return norm_path


def _app_dir():
    if is_macos():
        app_dir_path = Path.home() / 'Library' / 'Application Support' / Constants.ORGANIZATION_NAME / Constants.APPLICATION_NAME
    elif is_windows():
        app_dir_path = Path.home() / 'AppData' / 'Local' / Constants.ORGANIZATION_NAME / Constants.APPLICATION_NAME
    elif is_linux():
        # XDG_DATA_HOME is only used for the flatpak application since all the files are saved in the flatpak sandbox
        xdg_data = os.environ.get('XDG_DATA_HOME')
        if xdg_data:
            app_dir_path = Path(xdg_data) / Constants.ORGANIZATION_NAME / Constants.APPLICATION_NAME
        else:
            app_dir_path = Path.home() / '.local' / 'share' / Constants.ORGANIZATION_NAME / Constants.APPLICATION_NAME
    else:
        app_dir_path = BASE_DIR.resolve()

    if not app_dir_path.exists():
        app_dir_path.mkdir(parents=True)

    return app_dir_path


def _data_dir():
    data_dir_path = APP_DIR / 'data'
    if not data_dir_path.exists():
        data_dir_path.mkdir(parents=True)
    return data_dir_path


def _tmp_dir():
    if is_windows():
        tmp_dir_path = Path.home() / 'AppData' / 'Local' / 'Temp'
    else:
        tmp_dir_path = Path('/tmp')

    return tmp_dir_path


def _preference_dir():
    preference_dir_path = APP_DIR / 'config'
    if not preference_dir_path.exists():
        preference_dir_path.mkdir(parents=True)
    return preference_dir_path    


BASE_DIR = Path(os.path.dirname(__file__)) / '../..'
APP_DIR = _app_dir()
DATA_DIR = _data_dir()
TMP_DIR = _tmp_dir()
PREFERENCES_DIR = _preference_dir()



import sys
import logging

from PySide6.QtCore import QCoreApplication
from PySide6.QtGui import QGuiApplication, QIcon

from .utils.constants import Constants
from .utils.ui_utils import set_ui
from .ui.artemis import UIArtemis

from . import resources


def main():
    logger = logging.getLogger("peewee")
    logger.setLevel(logging.WARNING)
    logger.addHandler(logging.StreamHandler())

    set_ui()

    QCoreApplication.setOrganizationName(Constants.ORGANIZATION_NAME)
    QCoreApplication.setOrganizationDomain(Constants.ORGANIZATION_DOMAIN)
    QCoreApplication.setApplicationName(Constants.APPLICATION_NAME)

    app = QGuiApplication(sys.argv)

    icon_file_path = ":/data/images/artemis_icon.ico"
    app.setWindowIcon(QIcon(icon_file_path))

    UIArtemis()
    sys.exit(app.exec())


if __name__ == '__main__':
    main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

snake_case = 0x01
true_property = 0x02

all_feature_names = [
    "snake_case",
    "true_property",
]



from PySide6.QtQml import QQmlApplicationEngine
from PySide6.QtCore import QObject, Signal, Slot

from artemis.utils.sql_utils import ArtemisSIG
from artemis.utils.sys_utils import delete_file


class UIsignaleditor(QObject):
    # Python > QML Signals
    show_ui = Signal()
    load = Signal(str, list, bool)


    def __init__(self, parent):
        super().__init__()

        self._parent = parent

        self._engine = QQmlApplicationEngine()
        self._engine.load('qrc:/ui/SignalEditor.qml')
        self._window = self._engine.rootObjects()[0]

        self._connect()


    def _connect(self):
        # QML > Python connections
        self._window.saveParam.connect(self.save)
        self._window.deleteParam.connect(self.delete)

        # Python > QML connections
        self.show_ui.connect(self._window.show)
        self.load.connect(self._window.load)


    def load_signaleditor_ui(self, param_type, sig_param, is_new):
        """ Load all the details of the selected signal

        Args:
            param_type (str): Signal, Frequency, Bandwidth, Modulation, Mode,
                ACF, Location
            sig_param (list): a list formed as [id, value, description]
            is_new (bool): If true, the windows open in an empty state ready to 
                be compiled by the user. If false, the windows will open all the 
                current parameter for the loaded signal, for editing or deleting purposes.
        """
        if param_type == 'Signal' and not is_new:
            sig_param = [
                self._parent.loaded_sig.sig_id,
                self._parent.loaded_sig.name,
                self._parent.loaded_sig.description
            ]
        self.load.emit(param_type, sig_param, is_new)
        self.show_ui.emit()


    @Slot(str, list, bool)
    def save(self, param_type, data, is_new):
        """ Save new signal parameters or update the existing ones.
        """
        data = data.toVariant()

        if is_new:
            if param_type == 'Signal':
                self._parent.loaded_sig = ArtemisSIG(self._parent.loaded_db)
                self._parent.loaded_sig.insert_signal(data[1], data[2])
            elif param_type == 'Frequency':
                self._parent.loaded_sig.insert_frequency(int(data[1]), data[2])
            elif param_type == 'Bandwidth':
                self._parent.loaded_sig.insert_bandwidth(int(data[1]), data[2])
            elif param_type == 'Modulation':
                self._parent.loaded_sig.insert_modulation(data[1], data[2])        
            elif param_type == 'Mode':
                self._parent.loaded_sig.insert_mode(data[1], data[2])
            elif param_type == 'ACF':
                self._parent.loaded_sig.insert_acf(data[1], data[2])
            elif param_type == 'Location':
                self._parent.loaded_sig.insert_location(data[1], data[2])
        else:
            if param_type == 'Signal':
                self._parent.loaded_sig.update_signal(data[0], data[1], data[2])
            elif param_type == 'Frequency':
                self._parent.loaded_sig.update_frequency(data[0], int(data[1]), data[2])
            elif param_type == 'Bandwidth':
                self._parent.loaded_sig.update_bandwidth(data[0], int(data[1]), data[2])
            elif param_type == 'Modulation':
                self._parent.loaded_sig.update_modulation(data[0], data[1], data[2])        
            elif param_type == 'Mode':
                self._parent.loaded_sig.update_mode(data[0], data[1], data[2])
            elif param_type == 'ACF':
                self._parent.loaded_sig.update_acf(data[0], data[1], data[2])
            elif param_type == 'Location':
                self._parent.loaded_sig.update_location(data[0], data[1], data[2])

        self._parent.load_db(self._parent.loaded_db.db_dir_name)


    @Slot(str, int)
    def delete(self, param_type, id):
        """ Delete a signal parameter or the signal itself (with all the parameters and documents).
            All the entries in the documents table are automatically beign deleted due to 
            foreign-key cascade propagation
        """
        if param_type == 'Signal':
            self._parent.loaded_sig.delete_signal()
            self._parent.lock_audio_player.emit()
            for doc in self._parent.loaded_sig.documents:
                doc_file_name = f'{str(doc[0])}.{doc[1]}'
                doc_file_path = self._parent.loaded_db.media_dir / doc_file_name
                delete_file(doc_file_path)
        elif param_type == 'Frequency':
            self._parent.loaded_sig.delete_frequency(id)
        elif param_type == 'Bandwidth':
            self._parent.loaded_sig.delete_bandwidth(id)
        elif param_type == 'Modulation':
            self._parent.loaded_sig.delete_modulation(id)        
        elif param_type == 'Mode':
            self._parent.loaded_sig.delete_mode(id)
        elif param_type == 'ACF':
            self._parent.loaded_sig.delete_acf(id)
        elif param_type == 'Location':
            self._parent.loaded_sig.delete_location(id)

        self._parent.load_db(self._parent.loaded_db.db_dir_name)



import os
import platform
import subprocess
import hashlib
import tarfile

from shutil import rmtree, copyfile, unpack_archive
from pathlib import Path

from artemis.utils.constants import Messages


def is_windows() -> bool:
    """Checks if the current system OS is windows"""
    return platform.system() == 'Windows'

def is_macos() -> bool:
    """Checks if the current system OS is macos"""
    return platform.system() == 'Darwin'

def is_linux() -> bool:
    """Checks if the current system OS is linux"""
    return platform.system() == 'Linux'


def is_arm() -> bool:
    """Checks if the current system architecture is ARM-based."""
    arch = platform.machine().lower()
    return "arm" in arch or "aarch64" in arch

def is_x64() -> bool:
    """Checks if the current system architecture is x64-based."""
    arch = platform.machine().lower()
    return "x86_64" in arch or "amd64" in arch


def open_file(file_path, timeout=3):
    try:
        if is_windows():
            os.startfile(file_path)
        elif is_macos():
            subprocess.call(['open', file_path], timeout=timeout)
        elif is_linux():
            subprocess.call(['xdg-open', file_path], timeout=timeout)
        else:
            return
    except FileNotFoundError:
        raise Exception(Messages.FILE_NOT_FOUND_ERR_MSG)
    except Exception as e:
        raise Exception(Messages.GENERIC_ERROR_MSG.format(e))


def open_directory(directory, timeout=3):
    if is_windows():
        subprocess.call(['explorer', str(Path(directory))], timeout=timeout)
    elif is_macos():
        subprocess.call(['open', directory], timeout=timeout)
    elif is_linux():
        subprocess.call(['xdg-open', directory], timeout=timeout)
    else:
        return


def copy_file(src_file_path, dst_file_path):
    copyfile(src_file_path, dst_file_path)


def delete_dir(dir_path):
    if os.path.exists(dir_path):
        rmtree(dir_path)    


def delete_file(file_path):
    if os.path.exists(file_path):
        os.remove(file_path) 


def make_tar(save_path, origin_path):
    """ Create a tar archive from a folder

    Args:
        save_path (str): destination path exactly as provided by QUrl.toLocalFile()
        origin_path (Path): directory path of the folder to be archived
    """
    origin_str = str(origin_path.resolve())
    with tarfile.open(name=save_path, mode="w") as tar:
        tar.add(origin_str, arcname=".")


def unpack_tar(tar_path, destination_path):
    """ Unpack a tar archive in a folder

    Args:
        tar_path: path of the tar to be unpacked
        destination_path: path where the tar is extracted
    """
    unpack_archive(tar_path, destination_path, 'tar')


def match_hash(data, reference_hash):
    """ Check whether the checksum of 'data' match the reference one.

    Args:
        data (str): Path of the file to be checked
        reference_hash (str): Reference SHA-256 hash
    """
    if reference_hash is None:
        raise ValueError("ERROR: Invalid hash code.")

    code = hashlib.sha256()
    b = bytearray(128*1024)
    mv = memoryview(b)
    with open(data, 'rb', buffering=0) as f:
        while n := f.readinto(mv):
            code.update(mv[:n])

    return code.hexdigest() == reference_hash



from configparser import ConfigParser

from artemis.utils.path_utils import PREFERENCES_DIR, BASE_DIR
from artemis.utils.sys_utils import copy_file


class Config(ConfigParser):
    """ Custom configuration class derived from ConfigParser.
        Used to get value, set, save and remove any configuration from the conf file
    """

    def __init__(self, config_file_path, space_around_delimiters=False):
        super().__init__()
        self._config_file_path = config_file_path
        self.read(self._config_file_path)
        self._space_around_delimiters = space_around_delimiters

    def value(self, section, option, default_value):
        value = super().get(section, option, fallback=default_value)
        return value

    def set(self, section, option, value=None):
        if not self.has_section(section):
            self.add_section(section)
        super().set(section, option, value)
        self.save()

    def remove(self, section, option):
        super().remove_option(section, option)
        self.save()

    def save(self):
        with open(self._config_file_path, 'w') as f:
            self.write(f, space_around_delimiters=self._space_around_delimiters)


def merge_config_files(old_config_path, template_config_path):
    """ Merge two configuration files: if the old one lacks some
        sections or options from a comparison with a template,
        this function will add what is missing to the old conf file
    """
    old_config = ConfigParser()
    old_config.read(old_config_path)
    
    new_config = ConfigParser()
    new_config.read(template_config_path)
    
    for section in new_config.sections():
        if not old_config.has_section(section):
            old_config.add_section(section)
        for option in new_config.options(section):
            if not old_config.has_option(section, option):
                old_config.set(section, option, new_config.get(section, option))
    
    with open(old_config_path, 'w') as f:
        old_config.write(f)


def check_conf_file():
    """ Check the integrity of the used conf file.
        If it is not present it will add a copy to the PREF_DIR
        and if it is different in structure (different section/options)
        it will merge the conf file with the new template one
    """
    active_conf = (PREFERENCES_DIR / 'qtquickcontrols2.conf').resolve()
    template_conf = (BASE_DIR / 'config' / 'qtquickcontrols2.conf').resolve()

    if not active_conf.exists():
        copy_file(template_conf, active_conf)
    else:
        merge_config_files(active_conf, template_conf)


check_conf_file()
CONFIGURE_QT = Config((PREFERENCES_DIR / 'qtquickcontrols2.conf').resolve().as_posix())



from PySide6.QtQml import QQmlApplicationEngine
from PySide6.QtCore import QObject, Slot, Signal

from artemis.utils.config_utils import CONFIGURE_QT


class UIPreferences(QObject):
    # Python > QML Signals
    show_ui = Signal()
    load_material_accent = Signal(str)
    load_material_theme = Signal(str)
    load_scaling = Signal(str)
    load_autoload = Signal(int)


    def __init__(self, parent):
        super().__init__()

        self._parent = parent

        self._engine = QQmlApplicationEngine()
        self._engine.load('qrc:/ui/Preferences.qml')
        self._window = self._engine.rootObjects()[0]

        self._connect()


    def _connect(self):
        # QML > Python connections
        self._window.saveMaterialAccent.connect(self.save_material_accent)
        self._window.saveMaterialTheme.connect(self.save_material_theme)
        self._window.saveScaling.connect(self.save_scaling)
        self._window.saveAutoload.connect(self.save_autoload)

        # Python > QML connections
        self.show_ui.connect(self._window.show)
        self.load_material_accent.connect(self._window.loadMaterialAccent)
        self.load_material_theme.connect(self._window.loadMaterialTheme)
        self.load_scaling.connect(self._window.loadScaling)
        self.load_autoload.connect(self._window.loadAutoload)


    def load_preferences_ui(self):
        """ Loading all the initial preferences from the conf file to the UI
        """
        self.load_material_accent.emit(CONFIGURE_QT.value("Material", "Accent", "Green"))
        self.load_material_theme.emit(CONFIGURE_QT.value("Material", "Theme", "System"))
        self.load_scaling.emit(CONFIGURE_QT.value("Scaling", "factor", "1.00"))
        self.load_autoload.emit(int(CONFIGURE_QT.value("Database", "autoload", 0)))
        self.show_ui.emit()


    @Slot(str)
    def save_material_accent(self, material_accent):
        """ Saving material accent setting
        """
        CONFIGURE_QT.set("Material", "Accent", material_accent)


    @Slot(str)
    def save_material_theme(self, material_theme):
        """ Saving material theme setting
        """
        CONFIGURE_QT.set("Material", "Theme", material_theme)


    @Slot(str)
    def save_scaling(self, scaling):
        """ Saving UI scaling
        """
        CONFIGURE_QT.set("Scaling", "factor", scaling)


    @Slot(int)
    def save_autoload(self, autoload):
        """ Saving autoload setting
        """
        CONFIGURE_QT.set("Database", "autoload", str(autoload))



import os

from PySide6.QtCore import QUrl
from datetime import datetime

from artemis.utils.constants import Constants
from artemis.utils.generic_utils import format_frequency
from artemis.utils.path_utils import DATA_DIR

from artemis.model import (
    database, Info, Signals, Category, CategoryLabel, Frequency, Bandwidth, 
    Modulation, Mode, Location, Acf, Documents
)
from peewee import SqliteDatabase, IntegerField
from playhouse.migrate import SqliteMigrator, migrate


################################## MARK: ==== DATABASE ====
class ArtemisDB:
    def __init__(self, db_dir_name):
        self.db_dir_name = db_dir_name
        self.db_dir = DATA_DIR / db_dir_name
        self.sql_path = self.db_dir / Constants.SQL_NAME
        self.media_dir = self.db_dir / 'media'

        self.name = None
        self.date = None
        self.version = None
        self.editable = None
        self.is_sigid = None

        self.all_signals = []
        self.all_modulations = []
        self.all_locations = []
        self.all_category_labels = []
        self.all_since_versions = []

        self.count_signals = None
        self.count_docs = None
        self.count_images = None
        self.count_audio = None


        db = SqliteDatabase(self.sql_path, autoconnect=False)
        database.initialize(db)


    def load(self):
        """ Load all the initial data: info, a list of all signals (used for the main
            signals list, all the categories-locations-modulations (used to populate
            the filters combobox) 
        """
        self.load_info()
        self.load_stats()
        self._select_all_signals()
        self._select_all_category_labels()
        self._select_all_locations()
        self._select_all_modulations()
        self._select_all_since_versions()


    def load_info(self):
        """ Load the DB meta INFO from the table 'info'
        """
        with database:
            try:
                info_record = Info.select().first()
                if info_record:
                    self.name = info_record.name
                    self.date = info_record.date
                    self.version = info_record.version
                    self.editable = info_record.editable
                    self.is_sigid = True if self.editable == -1 else False
            except Exception as e:
                print(f"ERROR: {e}")


    def load_stats(self):
        with database:
            self.count_signals = Signals.select().count()
            self.count_docs = Documents.select().count()
            self.count_images = Documents.select().where(Documents.type == 'Image').count()
            self.count_audio = Documents.select().where(Documents.type == 'Audio').count()


    def _select_all_signals(self):
        """ Load a list of tuple for all signals. Each tuple (representing a signal)
            contains the SIG_ID, NAME adn DESCRITPION of the signal
        """
        with database:
            try:
                query = Signals.select(
                    Signals.sig_id, 
                    Signals.name, 
                    Signals.description
                ).dicts()
                self.all_signals = list(query)
            except Exception as e:
                print(f"ERROR: {e}")
                self.all_signals = []


    def _select_all_modulations(self):
        with database:
            try:
                query = Modulation.select(Modulation.value).distinct().order_by(Modulation.value).dicts()
                self.all_modulations = list(query)
            except Exception as e:
                print(f"ERROR: {e}")
                self.all_modulations = []


    def _select_all_locations(self):
        with database:
            try:
                query = Location.select(Location.value).distinct().order_by(Location.value).dicts()
                self.all_locations = list(query)
            except Exception as e:
                print(f"ERROR: {e}")
                self.all_locations = []


    def _select_all_category_labels(self):
        with database:
            try:
                query = CategoryLabel.select(
                    CategoryLabel.clb_id, 
                    CategoryLabel.value
                ).distinct().order_by(CategoryLabel.value).dicts()
                self.all_category_labels = list(query)
            except Exception as e:
                print(f"ERROR: {e}")
                self.all_category_labels = []


    def _select_all_since_versions(self):
        with database:
            try:
                query = (
                    Signals.select(Signals.since_version)
                    .where(Signals.since_version
                    .is_null(False))
                    .distinct()
                    .order_by(Signals.since_version.desc())
                    .scalars()
                )
                self.all_since_versions = list(query)
            except Exception as e:
                print(f"ERROR: {e}")
                self.all_since_versions = []

################################## MARK: MIGRATIONS
    def migrate_db(self):
        """ if models.py is changed and then the DB schema , hre goes al the necessary 
            migrations to assure the compatibility of older DB. MIGRATION 1 and 2 has been
            introduced during the ORM implementation.
        """
        with database:
            migrator = SqliteMigrator(database)

            # MIGRATION 1
            # Introduction of a new column called since_version in the signals table
            since_version_field = IntegerField(column_name='SINCE_VERSION', null=True)
            try:
                migrate(migrator.add_column('signals', 'SINCE_VERSION', since_version_field))
            except Exception:
                pass

            # MIGRATION 2
            # renamed the table category_label to categorylabel. The latter is peewee standard
            # nomencalture so the meta block in model.py is not necessary
            try:
                migrate(migrator.rename_table("category_label", "categorylabel"))
            except Exception:
                pass

################################## MARK: CREATE
    def create(self, name):
        """ Create new db in the data folder.
            The name of folder containing the new db has a unique id as name (db_dir_name).
        """
        os.makedirs(self.db_dir, exist_ok=True)
        os.makedirs(self.media_dir, exist_ok=True)

        with database:
            database.create_tables([
                Info, Signals, Acf, Bandwidth, CategoryLabel, 
                Category, Documents, Frequency, Location, Mode, Modulation
            ])

            Info.create(
                name=name,
                date=datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                version=1,
                editable=1
            )

        self.load()

################################## MARK: CRUD
    def rename(self, new_name):
        with database:
            Info.update(name=new_name).execute()
        self.name = new_name
    
    def delete_signal(self, sig_id):
        with database:
            Signals.delete().where(Signals.sig_id == sig_id).execute()
        self._select_all_signals()

    def insert_category_label(self, value):
        with database:
            CategoryLabel.create(value=value)
        self._select_all_category_labels()

    def update_category_label(self, clb_id, value):
        with database:
            CategoryLabel.update(value=value).where(CategoryLabel.clb_id == clb_id).execute()
        self._select_all_category_labels()

    def delete_category_label(self, clb_id):
        with database:
            CategoryLabel.delete().where(CategoryLabel.clb_id == clb_id).execute()
        self._select_all_category_labels()

################################## MARK: ==== SIGNAL ====
class ArtemisSIG():
    """ Main class of the object signal
    """
    def __init__(self, database):
        self.db = database
        self._signal = None

        self.sig_id = None
        self.name = None
        self.description = None
        self.url = None
        self.category = None
        self.frequency = None
        self.bandwidth = None
        self.modulation = None
        self.mode = None
        self.location = None
        self.acf = None
        self.since_version = None
        
        self.documents = None
        self.spectrum_path = None
        self.audio_path = None


    def load(self, sig_id):
        with database:
            self._signal = Signals.get(Signals.sig_id == sig_id)

        self.sig_id = sig_id
        
        self._select_signals()
        self._select_category()
        self._select_frequency()
        self._select_bandwidth()
        self._select_modulation()
        self._select_mode()
        self._select_location()
        self._select_acf()
        self.select_documents()


    @property
    def summary(self):
        return {
            'name': self.name,
            'description': self.description,
            'url': self.url,
            'since_version': self.since_version,
            'category': self.category,
            'frequency': self.frequency,
            'bandwidth': self.bandwidth,
            'modulation': self.modulation,
            'mode': self.mode,
            'location': self.location,
            'acf': self.acf,
            'spectrum_path': self.spectrum_path,
            'audio_path': self.audio_path,
            'all_category': self.db.all_category_labels
        }

################################## MARK: CRUD > SELECT
    def _select_signals(self):
        with database:
            self.name = self._signal.name
            self.description = self._signal.description
            self.url = self._signal.url
            self.since_version = self._signal.since_version


    def _select_category(self):
        with database:
            query = (Category
                    .select(Category.cat_id, CategoryLabel.clb_id, CategoryLabel.value)
                    .join(CategoryLabel)
                    .where(Category.sig == self._signal))
            
            self.category = [[c.cat_id, c.clb.clb_id, c.clb.value] for c in query]


    def _select_frequency(self):
        with database:
            query = self._signal.frequencies.order_by(Frequency.value)

            self.frequency = [
                [f.freq_id, f.value, f.description, format_frequency(f.value)] 
                for f in query
            ]


    def _select_bandwidth(self):
        with database:
            query = self._signal.bandwidths.order_by(Bandwidth.value)
            self.bandwidth = [
                [b.band_id, b.value, b.description, format_frequency(b.value)] 
                for b in query
            ]


    def _select_acf(self):
        with database:
            self.acf = [[a.acf_id, a.value, a.description] for a in self._signal.acfs]


    def _select_modulation(self):
        with database:
            self.modulation = [[m.mdl_id, m.value, m.description] for m in self._signal.modulations]


    def _select_mode(self):
        with database:
            self.mode = [[m.mod_id, m.value, m.description] for m in self._signal.modes]


    def _select_location(self):
        with database:
            self.location = [[loc.loc_id, loc.value, loc.description] for loc in self._signal.locations]


    def select_documents(self):
        with database:
            docs = self._signal.documents
            self.documents = [[d.doc_id, d.extension, d.name, d.description, d.type, d.preview] for d in docs]

            default_spectrum = [d for d in docs if d.type == 'Image' and d.preview == 1]
            default_audio = [d for d in docs if d.type == 'Audio' and d.preview == 1]

            if default_spectrum:
                spec = default_spectrum[0]
                default_spectrum_filename = f"{spec.doc_id}.{spec.extension}"
                full_path = self.db.media_dir / default_spectrum_filename
                self.spectrum_path = QUrl.fromLocalFile(str(full_path.resolve()))
            else:
                self.spectrum_path = 'qrc:///data/images/spectrum_not_available.svg'

            if default_audio:
                aud = default_audio[0]
                default_audio_filename = f"{aud.doc_id}.{aud.extension}"
                full_path = self.db.media_dir / default_audio_filename
                self.audio_path = QUrl.fromLocalFile(str(full_path.resolve()))
            else:
                self.audio_path = ''

################################## MARK: CRUD > UPDATE
    def update_signal(self, sig_id, value, description):
        with database:
            Signals.update(name=value, description=description).where(Signals.sig_id == sig_id).execute()


    def update_frequency(self, freq_id, value, description):
        with database:
            Frequency.update(value=value, description=description).where(Frequency.freq_id == freq_id).execute()


    def update_bandwidth(self, band_id, value, description):
        with database:
            Bandwidth.update(value=value, description=description).where(Bandwidth.band_id == band_id).execute()


    def update_modulation(self, modu_id, value, description):
        with database:
            Modulation.update(value=value, description=description).where(Modulation.mdl_id == modu_id).execute()


    def update_mode(self, mode_id, value, description):
        with database:
            Mode.update(value=value, description=description).where(Mode.mod_id == mode_id).execute()


    def update_acf(self, acf_id, value, description):
        with database:
            Acf.update(value=value, description=description).where(Acf.acf_id == acf_id).execute()


    def update_location(self, loc_id, value, description):
        with database:
            Location.update(value=value, description=description).where(Location.loc_id == loc_id).execute()


    def update_documents(self, doc_id, name, description, type, is_preview):
        with database:
            Documents.update(
                name=name, 
                description=description, 
                type=type, 
                preview=is_preview
            ).where(Documents.doc_id == doc_id).execute()

################################## MARK: CRUD > INSERT
    def insert_signal(self, value, description):
        # Returns the created object, if necessary
        with database:
            return Signals.create(name=value, description=description)


    def insert_frequency(self, value, description):
        with database:
            Frequency.create(sig=self.sig_id, value=value, description=description)


    def insert_bandwidth(self, value, description):
        with database:
            Bandwidth.create(sig=self.sig_id, value=value, description=description)


    def insert_modulation(self, value, description):
        with database:
            Modulation.create(sig=self.sig_id, value=value, description=description)


    def insert_mode(self, value, description):
        with database:
            Mode.create(sig=self.sig_id, value=value, description=description)


    def insert_acf(self, value, description):
        with database:
            Acf.create(sig=self.sig_id, value=value, description=description)


    def insert_location(self, value, description):
        with database:
            Location.create(sig=self.sig_id, value=value, description=description)


    def insert_category(self, clb_id):
        with database:
            Category.create(sig=self.sig_id, clb=clb_id)


    def insert_document(self, doc_lst):
        with database:
            new_doc = Documents.create(
                sig=self.sig_id,
                name=doc_lst[2],
                description=doc_lst[3],
                extension=doc_lst[1],
                type=doc_lst[4],
                preview=doc_lst[5]
            )
        return new_doc.doc_id

################################## MARK: CRUD > DELETE
    def delete_signal(self):
        with database:
            Signals.delete().where(Signals.sig_id == self.sig_id).execute()


    def delete_frequency(self, freq_id):
        with database:
            Frequency.delete().where(Frequency.freq_id == freq_id).execute()


    def delete_bandwidth(self, band_id):
        with database:
            Bandwidth.delete().where(Bandwidth.band_id == band_id).execute()


    def delete_modulation(self, modu_id):
        with database:
            Modulation.delete().where(Modulation.mdl_id == modu_id).execute()


    def delete_mode(self, mode_id):
        with database:
            Mode.delete().where(Mode.mod_id == mode_id).execute()


    def delete_acf(self, acf_id):
        with database:
            Acf.delete().where(Acf.acf_id == acf_id).execute()


    def delete_location(self, loc_id):
        with database:
            Location.delete().where(Location.loc_id == loc_id).execute()


    def delete_document(self, doc_id):
        with database:
            Documents.delete().where(Documents.doc_id == doc_id).execute()


    def delete_category(self, cat_id):
        with database:
            Category.delete().where(Category.cat_id == cat_id).execute()



from PySide6.QtQml import QQmlApplicationEngine
from PySide6.QtCore import QObject, Signal, Slot

from artemis.utils.path_utils import *
from artemis.utils.generic_utils import *


class UIcategoryeditor(QObject):
    # Python > QML Signals
    show_ui = Signal()
    load = Signal(list)


    def __init__(self, parent):
        super().__init__()

        self._parent = parent

        self._engine = QQmlApplicationEngine()
        self._engine.load('qrc:/ui/CategoryEditor.qml')
        self._window = self._engine.rootObjects()[0]

        self._connect()


    def _connect(self):
        # QML > Python connections
        self._window.saveParam.connect(self.save)
        self._window.deleteParam.connect(self.delete)

        # Python > QML connections
        self.show_ui.connect(self._window.show)
        self.load.connect(self._window.loadList)


    def load_cateditor_ui(self):
        """ Load the list with existing category tags and show the UI
        """
        all_cat = self._parent.loaded_db.all_category_labels
        self.load.emit(all_cat)
        self.show_ui.emit()


    @Slot(list, bool)
    def save(self, data, is_new):
        """ Save new category tag or update the existing ones.
        """
        data = data.toVariant()

        if is_new:
            self._parent.loaded_db.insert_category_label(data[0])
        else:
            self._parent.loaded_db.update_category_label(data[1], data[0])

        self._parent.load_db(self._parent.loaded_db.db_dir_name)
        self.load_cateditor_ui()


    @Slot(int)
    def delete(self, clb_id):
        """ Delete a database category tag.
            All the entries in the documents table are automatically beign deleted due to 
            foreign-key cascade propagation
        """
        self._parent.loaded_db.delete_category_label(clb_id)
        self._parent.load_db(self._parent.loaded_db.db_dir_name)
        self.load_cateditor_ui()



import os

from PySide6.QtQml import QQmlApplicationEngine
from PySide6.QtCore import QObject, Signal, Slot

from artemis.utils.path_utils import DATA_DIR
from artemis.utils.generic_utils import parse_date
from artemis.utils.sql_utils import ArtemisDB
from artemis.utils.sys_utils import delete_dir


class UIdbmanager(QObject):
    # Python > QML Signals
    show_ui = Signal()
    close_ui = Signal()
    populate_db_list = Signal(list, str)


    def __init__(self, parent):
        super().__init__()

        self._parent = parent

        self._engine = QQmlApplicationEngine()
        self._engine.load('qrc:/ui/DbManager.qml')
        self._window = self._engine.rootObjects()[0]

        self._connect()


    def _connect(self):
        # QML > Python connections
        self._window.loadDB.connect(self.load_db)
        self._window.deleteDB.connect(self.delete_db)
        self._window.renameDB.connect(self.rename_db)

        # Python > QML connections
        self.show_ui.connect(self._window.show)
        self.close_ui.connect(self._window.close)
        self.populate_db_list.connect(self._window.loadList)


    def load_dbmanager_ui(self):
        self.load_local_db_list()
        self.show_ui.emit()


    def load_local_db_list(self):
        """ Scan for all the valid DBs in the data folder and show them on the list
            Send to the qml also the loaded DB name, if exist. This is usefull to
            render the badge 'LOADED' when a DB is selected from the list
        """
        db_param = []
        valid_db_list = self.scan_db_dir()
        dir_name_loaded_db = self._parent.loaded_db.db_dir_name if self._parent.loaded_db is not None else ""

        for db in valid_db_list:
            db_param.append(
                {
                    'name': db.name,
                    'version': db.version,
                    'date': parse_date(db.date),
                    'db_dir_name': db.db_dir_name,
                    'documents_n': db.count_docs,
                    'signals_n': db.count_signals,
                    'images_n': db.count_images,
                    'audio_n': db.count_audio
                }
            )

        self.populate_db_list.emit(db_param, dir_name_loaded_db)


    def load_db(self, db_dir_name):
        """ Load the selected DB (from the DB Manager list) in the main artemis window
        """
        self._parent.load_db(db_dir_name)
        self.close_ui.emit()


    @Slot(str)
    def delete_db(self, db_dir_name):
        """ Delete the DB folder.
            Clear the main UI if the database to be deleted is the selected one 
        """
        if self._parent.loaded_db is not None:
            if self._parent.loaded_db.db_dir_name == db_dir_name:
                self._parent.lock_menu.emit(True, False)
                self._parent.clear_list.emit()
                self._parent.clear_signal_page.emit()
                self._parent.loaded_db = None
                self._parent.loaded_sig = None

        delete_dir(DATA_DIR / db_dir_name)
        self.load_local_db_list()


    @Slot(str, str)
    def rename_db(self, db_dir_name, new_name):
        """ Rename db in the data folder
        """
        database = ArtemisDB(db_dir_name)
        database.rename(new_name)
        self.load_local_db_list()


    def scan_db_dir(self):
        """ Scans the data directory for valid databases and
            return a dictionary containing only the valid ones.
            Returns a list of objects (dbs)
        """
        valid_db_list = []
        db_dirs = next(os.walk(DATA_DIR))[1]

        for db_dir_name in db_dirs:
            try:
                database = ArtemisDB(db_dir_name)
                database.migrate_db()
                database.load_info()
                database.load_stats()
                valid_db_list.append(database)
            except Exception:
                continue

        return valid_db_list


    def get_latest_local_sigid_db(self):
        """ Return the newest valid local sigID database.
            Returns None if no valid sigID database is found.
        """
        valid_dbs = self._parent.dbmanager.scan_db_dir()
        sig_id_dbs = [db for db in valid_dbs if db.is_sigid]

        if len(sig_id_dbs) != 0:
            sig_id_latest = max(sig_id_dbs, key=lambda x: x.version)
            return sig_id_latest
        else:
            return None



import os

from artemis.utils.sys_utils import is_windows, is_linux
from artemis.utils.config_utils import CONFIGURE_QT


def set_ui():
    # Style and aspect
    os.environ['QT_QUICK_CONTROLS_STYLE'] = CONFIGURE_QT.value('Controls', 'style', 'Material')
    os.environ['QT_QUICK_CONTROLS_MATERIAL_VARIANT'] = CONFIGURE_QT.value('Material', 'variant', 'Dense')
    os.environ['QT_QUICK_CONTROLS_MATERIAL_THEME'] = CONFIGURE_QT.value('Material', 'theme', 'System')
    os.environ['QT_QUICK_CONTROLS_MATERIAL_ACCENT'] = CONFIGURE_QT.value('Material', 'accent', 'Green')

    # Scaling
    os.environ['QT_SCALE_FACTOR'] = CONFIGURE_QT.value('Scaling', 'factor', '1.00')

    # GUI Backend
    if is_windows():
        os.environ['QSG_RHI_BACKEND'] = 'opengl'

    if is_linux():
        os.environ['QT_QPA_PLATFORM'] = 'wayland;xcb'

    # Optimizations and workarounds
    os.environ['QT_ENABLE_GLYPH_CACHE_WORKAROUND'] = '1'
    os.environ['QML_USE_GLYPHCACHE_WORKAROUND'] = '1'

    # Debug and loggings
    debug = CONFIGURE_QT.value('Develop', 'debug', '0')

    os.environ['QT_DEBUG_PLUGINS'] = CONFIGURE_QT.value('Develop', 'debug_plugin', '0')

    if debug != '1':
        os.environ["QT_LOGGING_RULES"] = "qt.multimedia.ffmpeg*=false"



from PySide6.QtQml import QQmlApplicationEngine
from PySide6.QtCore import QObject, Slot, Signal, QUrl, QSaveFile, QDir, QIODevice
from PySide6.QtNetwork import QNetworkReply, QNetworkRequest, QNetworkAccessManager

from artemis.utils.constants import Messages


class UIDownloader(QObject):
    # Python > QML Signals
    show_ui = Signal()
    close_ui = Signal()
    update_progress_bar = Signal(int, int)
    set_indeterminate_bar = Signal()
    update_status = Signal(str)
    finished = Signal()


    def __init__(self, parent):
        super().__init__()

        self._parent = parent

        self._engine = QQmlApplicationEngine()
        self._engine.load('qrc:/ui/Downloader.qml')
        self._window = self._engine.rootObjects()[0]

        self.file_url = None
        self.file_size = None
        self.dest_file = None
        self.file = None
        self.manager = QNetworkAccessManager(self)
        self.reply = None

        self._connect()


    def _connect(self):
        # QML > Python connections
        self._window.abortRequested.connect(self.on_abort)

        # Python > QML connections
        self.show_ui.connect(self._window.show)
        self.close_ui.connect(self._window.close)
        self.update_progress_bar.connect(self._window.updateProgressBar)
        self.set_indeterminate_bar.connect(self._window.setIndeterminateBar)
        self.update_status.connect(self._window.updateStatus)


    def on_start(self, url, save_path):
        """ Start the download process using the specified URL

        Args:
            url (str): url from where download the file
            save_path (str): path where to save the downloaded file
        """
        self._clear_ui()
        self.show_ui.emit()

        self.file_url = QUrl(url)
        self.file_size = None
        dest_path = QDir(save_path)
        self.dest_file = dest_path.filePath(self.file_url.fileName())
        self.file = QSaveFile(self.dest_file)

        if self.file.open(QIODevice.WriteOnly):
            # Start a GET HTTP request
            self.reply = self.manager.get(QNetworkRequest(self.file_url))
            self.reply.metaDataChanged.connect(self.on_metadata_changed)
            self.reply.downloadProgress.connect(self.on_progress)
            self.reply.finished.connect(self.on_finished)
            self.reply.readyRead.connect(self.on_ready_read)
            self.reply.errorOccurred.connect(self.on_error)
        else:
            self.close_ui.emit()
            self.show_popup_error(self.file.errorString())

    @Slot()
    def on_metadata_changed(self):
        """ Read the Content-Length header once the reply's metadata
            becomes available. If the header is missing, leave
            file_size as None and set the progress bar to
            'indeterminate' mode instead.
        """
        if not self.reply:
            return

        content_length = self.reply.header(QNetworkRequest.ContentLengthHeader)
        if content_length is not None:
            self.file_size = int(content_length)
        else:
            self.set_indeterminate_bar.emit()

    @Slot()
    def on_abort(self):
        """ Stop the download when user presses the abort button
        """
        if self.reply:
            self.reply.abort()


    @Slot()
    def on_ready_read(self):
        """ Write available bytes to the file
        """
        if self.reply:
            if self.reply.error() == QNetworkReply.NoError:
                self.file.write(self.reply.readAll())


    @Slot()
    def on_finished(self):
        """ Finalize the download process and, if no errors
            occurs, emits the finished signal usefull for
            a callback
        """
        if not self.reply:
            return

        has_error = self.reply.error() != QNetworkReply.NoError

        if self.file:
            if not has_error:
                self.file.commit()
            else:
                self.file.cancelWriting()

        self.reply.deleteLater()
        self.reply = None
        
        if not has_error:
            self.finished.emit()

        self.close_ui.emit()


    @Slot(int, int)
    def on_progress(self, bytesReceived: int, bytesTotal: int):
        """ Update progress bar and status label

            Note: file_size (from the Content-Length header) may not be
            populated yet on the very first call, since metaDataChanged
            and the first downloadProgress signal can race. In that case
            we fall back to bytesTotal for this tick only; file_size
            takes over on subsequent calls once available.
        """
        total = self.file_size if self.file_size is not None else bytesTotal

        if total > 0:
            self.update_status.emit(f"{bytesReceived/10**6:.1f} Mb / {total/10**6:.1f} Mb")
            self.update_progress_bar.emit(bytesReceived, total)
        else:
            self.update_status.emit(f"{bytesReceived/10**6:.1f} Mb")


    @Slot(QNetworkReply.NetworkError)
    def on_error(self, code: QNetworkReply.NetworkError):
        """ Show a message if an error happen during download
        """
        if code == QNetworkReply.OperationCanceledError:
            return

        if self.reply:
            self.close_ui.emit()
            self.show_popup_error(
                self.reply.errorString()
            )

    def _clear_ui(self):
        self.update_progress_bar.emit(0, 0)
        self.update_status.emit('')


    def show_popup_error(self, error_msg):
        self._parent.dialog_popup(
            Messages.DIALOG_TYPE_ERROR,
            Messages.GENERIC_ERROR,
            Messages.GENERIC_ERROR_MSG.format(error_msg)
        )



from PySide6.QtCore import QObject, Slot, Signal

from peewee import ModelSelect

from artemis.model import (
    database, Signals, Category, CategoryLabel, Frequency, Bandwidth, 
    Modulation, Location, Acf
)


class FiltersManager(QObject):
    # Python > QML Signals
    populate_filter_list = Signal(list)

    def __init__(self, parent):
        super().__init__()
        self._parent = parent

        self._window = self._parent._window
        self.location_filter = self._window.findChild(QObject, "locationDialogObj")
        self.category_filter = self._window.findChild(QObject, "categoryDialogObj")
        self.modulation_filter = self._window.findChild(QObject, "moculationDialogObj")

        self._connect()


    def _connect(self):
        # QML > Python connections
        self._window.applyFilter.connect(self.apply_filter)

        # Python > QML connections
        self.populate_filter_list.connect(self._window.populateFilterLists)


    def load_filter_lists(self):
        locations = self._parent.loaded_db.all_locations
        modulations = self._parent.loaded_db.all_modulations
        categories = self._parent.loaded_db.all_category_labels
        since_versions = self._parent.loaded_db.all_since_versions

        self.populate_filter_list.emit([{
            'location': locations,
            'modulation': modulations,
            'category': categories,
            'since_version': since_versions
        }])


    @Slot(dict)
    def apply_filter(self, filter_status: dict):
        """ Update the signal list according to the selected filters.

        Args:
            filter_status (dict): dictionary containing the active filters with all
            the details to generate a search query
        """
        filter_status = filter_status.toVariant()

        if self._parent.loaded_db is not None:
            # If filter_status is empty then reload all signals back
            if not filter_status:
                self._parent.loaded_db._select_all_signals()
                self._parent.clear_signal_page.emit()
                self._parent.populate_sig_list.emit(self._parent.loaded_db.all_signals)
                self._parent.bottom_info_bar(
                    f"{self._parent.loaded_db.name} v{self._parent.loaded_db.version} | {self._parent.loaded_db.count_signals} signals",
                    "info"
                )
                return

            try:
                filtered_signals = self._get_filtered_signals_query(filter_status)

                self._parent.loaded_db.all_signals = filtered_signals

                self._parent.clear_signal_page.emit()
                self._parent.populate_sig_list.emit(filtered_signals)

                total_signals = len(filtered_signals)
                self._parent.bottom_info_bar(
                    f"{self._parent.loaded_db.name} v{self._parent.loaded_db.version} | {total_signals} ({self._parent.loaded_db.count_signals}) signals | FILTERS ACTIVE",
                    "warning"
                )

            except Exception as e:
                self._parent.bottom_info_bar(f"Error applying filters: {e}", "danger")


    def _get_filtered_signals_query(self, filter_status: dict) -> ModelSelect:
        """ Generete query using the applied filters summarized in the dictionary
        """
        with database:
            query = (Signals.select(Signals.sig_id, Signals.name, Signals.description))

            conditions = []
            joined_models = set()

            def apply_join(model):
                nonlocal query
                # Check if the model has not been joined yet to avoid redundant joins
                if model not in joined_models:
                    query = query.switch(Signals).join(model)
                    # Mark the model as joined by adding it to the set
                    joined_models.add(model)

            # Iterate through each filter key and value provided in the dictionary
            for key, val in filter_status.items():
                if not val:
                    continue

                if key == 'frequency':
                    apply_join(Frequency)
                    conditions.append(Frequency.value.between(val['lower_band'], val['upper_band']))

                elif key == 'bandwidth':
                    apply_join(Bandwidth)
                    conditions.append(Bandwidth.value.between(val['lower_band'], val['upper_band']))

                elif key == 'acf':
                    apply_join(Acf)
                    conditions.append(Acf.value.between(val['lower_band'], val['upper_band']))

                elif key == 'modulation':
                    apply_join(Modulation)
                    conditions.append(Modulation.value.in_(val))

                elif key == 'location':
                    apply_join(Location)
                    conditions.append(Location.value.in_(val))

                elif key == 'since_version':
                    conditions.append(Signals.since_version.in_(val))

                elif key == 'category':
                    if Category not in joined_models:
                        query = query.switch(Signals).join(Category).join(CategoryLabel)
                        joined_models.add(Category)
                    conditions.append(CategoryLabel.value.in_(val))

            if conditions:
                # Unpack and apply all collected conditions to the query using an AND logic
                query = query.where(*conditions)

            query = query.distinct().dicts()

            return list(query)



import uuid
import requests

from packaging.version import Version

from artemis.utils.constants import Constants, Messages
from artemis.utils.sys_utils import is_windows, is_linux, is_macos, is_arm, is_x64, delete_file, match_hash, unpack_tar, open_file
from artemis.utils.path_utils import DATA_DIR, TMP_DIR
from artemis.utils.sql_utils import ArtemisDB

class UpdateManager:
    """ Class used to manage DB and software updates
    """

    def __init__(self, parent):
        self._parent = parent

        self.db_update = None
        self.art_update = None

        self.remote_db_url = None
        self.remote_db_hash = None
        self.remote_db_version = None
        self.remote_db_size = None
        self.remote_db_file_name = None

        self.remote_artemis_version = None
        self.remote_artemis_url = None
        self.remote_artemis_file_name = None

        self.update()


    def _get_meta(self):
        latest_json = self.fetch_remote_json(Constants.LATEST_VERSION_URL)

        if not latest_json:
            self.db_update = False
            self.art_update = False
            return

        local_db = self._parent.dbmanager.get_latest_local_sigid_db()
        remote_db = latest_json['sigID_DB']

        self.remote_db_version = remote_db['version']
        self.remote_db_url = remote_db['url']
        self.remote_db_hash = remote_db['sha256_hash']
        self.remote_db_size = remote_db['total_bytes']
        self.remote_db_file_name = self.remote_db_url.split('/')[-1]

        if is_windows():
            if is_x64():
                os_name = 'windows'
            elif is_arm():
                os_name = 'windows-ARM64'
            self.remote_artemis_version = latest_json[os_name]['version']
            self.remote_artemis_url = latest_json[os_name]['url']
        elif is_linux():
            self.remote_artemis_version = latest_json['linux']['version']
            self.remote_artemis_url = latest_json['linux']['url']
        elif is_macos():
            self.remote_artemis_version = latest_json['mac']['version']
            self.remote_artemis_url = latest_json['mac']['url']

        self.remote_artemis_file_name = self.remote_artemis_url.split('/')[-1]

        if Version(self.remote_artemis_version) > Version(Constants.APPLICATION_VERSION):
            self.art_update = True
        else:
            self.art_update = False         

        if local_db:
            if self.remote_db_version > local_db.version:
                self.db_update = True
            else:
                self.db_update = False
        else:
            self.db_update = None


    def update(self, show_popup=False):
        """ Main updating routine

            Args:
                show_popup (bool, optional): If false, suppress any error message
                Defaults to False (to avoid error if the program is used offline)
        """
        try:
            self._get_meta()

            if self.db_update:
                self._show_popup_db_update()
            elif self.db_update is None:
                self._show_popup_initial_db_download()

            if self.art_update:
                self._show_popup_art_update()

            has_updates = bool(self.art_update or self.db_update or self.db_update is None)

            if show_popup and not has_updates:
                self._show_popup_up_to_date()

            self._parent.set_update_available(has_updates)

        except requests.exceptions.RequestException as e:
            self.db_update = False
            self.art_update = False

            if show_popup:
                self._parent.dialog_popup(
                    Messages.DIALOG_TYPE_ERROR,
                    Messages.NO_CONNECTION,
                    Messages.NO_CONNECTION_MSG.format(e)
                )


    def fetch_remote_json(self, url):
        """ Fetches the remote json from a url
        """
        response = requests.get(url, timeout=10)
        response.raise_for_status()
        return response.json()


    def download_db(self):
        """ Open the downloader and download the sigID database in the 
            TMP_DIR folder. After a succesfull download the callback function
            from the downloader is post_download_db
        """
        self._parent.downloader.finished.connect(self.post_download_db)
        self._parent.downloader.on_start(
            self.remote_db_url,
            TMP_DIR
        )


    def post_download_db(self):
        """ After a succesfull DB download, this function check the hash
            for possible corrupted data, delete old sigID DB and extract
            the new one
        """
        latest_db_tar_path = TMP_DIR / self.remote_db_file_name
        if match_hash(latest_db_tar_path, self.remote_db_hash):
            db_dir_name = str(uuid.uuid4())
            unpack_tar(latest_db_tar_path, DATA_DIR / db_dir_name)
            # Apply migrations if necessary
            db = ArtemisDB(db_dir_name)
            db.migrate_db()
            ####
            self._parent.load_db(db_dir_name)
            self._parent.set_update_available(False)
            self._show_popup_db_download_complete()
        else:
            self._show_popup_db_hash_failed()
        delete_file(latest_db_tar_path)


    def download_artemis(self):
        """ Open the downloader and download Artemis in the 
            TMP_DIR folder. After a succesfull download the callback function
            from the downloader is post_download_artemis
        """
        self._parent.downloader.finished.connect(self.post_download_artemis)
        self._parent.downloader.on_start(
            self.remote_artemis_url,
            TMP_DIR
        )


    def post_download_artemis(self):
        """ After a succesfull Artemis download, this open the installer
            and close the application
        """
        if is_windows():
            open_file(TMP_DIR / self.remote_artemis_file_name)
            self._parent.close_ui.emit()


    def _show_popup_db_update(self):
        """ Prompts the user to download the updated version of the database
        """
        self._parent.dialog_download_db(
            Messages.DIALOG_TYPE_WARN,
            Messages.DB_NEW_VER,
            Messages.DB_NEW_VER_MSG.format(self.remote_db_version)
        )


    def _show_popup_art_update(self):
        """ Alerts the user of a new version of Artemis.
            Windows - asks to download with automatic update
            Linux, macOS - redirects to GitHub page
        """
        if is_windows():
            self._parent.dialog_update_artemis(
                Messages.DIALOG_TYPE_QUEST,
                Messages.ART_NEW_VER,
                Messages.ART_NEW_VER_AUTO_MSG.format(self.remote_artemis_version),
                True
            )
        else:
            self._parent.dialog_update_artemis(
                Messages.DIALOG_TYPE_QUEST,
                Messages.ART_NEW_VER,
                Messages.ART_NEW_VER_MANUAL_MSG.format(self.remote_artemis_version),
                False
            )


    def _show_popup_up_to_date(self):
        """ Notifies the user that the database is up to date
        """
        self._parent.dialog_popup(
            Messages.DIALOG_TYPE_INFO,
            Messages.UP_TO_DATE,
            Messages.UP_TO_DATE_MSG
        )


    def _show_popup_initial_db_download(self):
        """ Prompts the user to download the database for the first time
        """
        self._parent.dialog_download_db(
            Messages.DIALOG_TYPE_QUEST,
            Messages.NO_DB_DETECTED,
            Messages.NO_DB_DETECTED_MSG
        )


    def _show_popup_db_download_complete(self):
        """ DB has been succesfully downloaded
        """
        self._parent.dialog_popup(
            Messages.DIALOG_TYPE_INFO,
            Messages.GENERIC_SUCCESS,
            Messages.DB_DOWNLOAD_SUCCESS_MSG
        )


    def _show_popup_db_hash_failed(self):
        """ Notify the user after detection of a corrupted database
        """
        self._parent.dialog_popup(
            Messages.DIALOG_TYPE_ERROR,
            Messages.DB_CORRUPTED,
            Messages.DB_CORRUPTED_MSG
        )



from peewee import (
    SqliteDatabase, 
    Proxy, 
    Model, 
    TextField, 
    AutoField, 
    ForeignKeyField, 
    FloatField, 
    IntegerField
)

database: SqliteDatabase = Proxy()

class BaseModel(Model):
    class Meta:
        database = database


class Info(BaseModel):
    date = TextField(column_name='DATE', null=True)
    editable = IntegerField(column_name='EDITABLE', null=True)
    name = TextField(column_name='NAME', null=True)
    version = IntegerField(column_name='VERSION', null=True)

    class Meta:
        primary_key = False


class Signals(BaseModel):
    sig_id = AutoField(column_name='SIG_ID')
    description = TextField(column_name='DESCRIPTION', null=True)
    name = TextField(column_name='NAME', null=True)
    url = TextField(column_name='URL', null=True)
    since_version = IntegerField(column_name='SINCE_VERSION', null=True)


class Acf(BaseModel):
    acf_id = AutoField(column_name='ACF_ID')
    description = TextField(column_name='DESCRIPTION', null=True)
    sig = ForeignKeyField(Signals, column_name='SIG_ID', null=True, backref='acfs')
    value = FloatField(column_name='VALUE', null=True)


class Bandwidth(BaseModel):
    band_id = AutoField(column_name='BAND_ID')
    description = TextField(column_name='DESCRIPTION', null=True)
    sig = ForeignKeyField(Signals, column_name='SIG_ID', null=True, backref='bandwidths')
    value = IntegerField(column_name='VALUE', null=True)


class CategoryLabel(BaseModel):
    clb_id = AutoField(column_name='CLB_ID')
    value = TextField(column_name='VALUE', null=True)


class Category(BaseModel):
    cat_id = AutoField(column_name='CAT_ID')
    clb = ForeignKeyField(CategoryLabel, field='clb_id', column_name='CLB_ID', null=True, backref='categories_labels')
    sig = ForeignKeyField(Signals, column_name='SIG_ID', null=True, backref='categories_signals')


class Documents(BaseModel):
    doc_id = AutoField(column_name='DOC_ID')
    description = TextField(column_name='DESCRIPTION', null=True)
    extension = TextField(column_name='EXTENSION', null=True)
    name = TextField(column_name='NAME', null=True)
    preview = IntegerField(column_name='PREVIEW', null=True)
    sig = ForeignKeyField(Signals, column_name='SIG_ID', null=True, backref='documents')
    type = TextField(column_name='TYPE', null=True)


class Frequency(BaseModel):
    freq_id = AutoField(column_name='FREQ_ID')
    description = TextField(column_name='DESCRIPTION', null=True)
    sig = ForeignKeyField(Signals, column_name='SIG_ID', null=True, backref='frequencies')
    value = IntegerField(column_name='VALUE', null=True)


class Location(BaseModel):
    loc_id = AutoField(column_name='LOC_ID')
    description = TextField(column_name='DESCRIPTION', null=True)
    sig = ForeignKeyField(Signals, column_name='SIG_ID', null=True, backref='locations')
    value = TextField(column_name='VALUE', null=True)


class Mode(BaseModel):
    mod_id = AutoField(column_name='MOD_ID')
    description = TextField(column_name='DESCRIPTION', null=True)
    sig = ForeignKeyField(Signals, column_name='SIG_ID', null=True, backref='modes')
    value = TextField(column_name='VALUE', null=True)


class Modulation(BaseModel):
    mdl_id = AutoField(column_name='MDL_ID')
    description = TextField(column_name='DESCRIPTION', null=True)
    sig = ForeignKeyField(Signals, column_name='SIG_ID', null=True, backref='modulations')
    value = TextField(column_name='VALUE', null=True)



import uuid

from PySide6.QtQml import QQmlApplicationEngine
from PySide6.QtCore import QObject, Slot, Signal, QUrl

from artemis.utils.constants import Constants, Messages
from artemis.utils.sys_utils import open_directory, make_tar, unpack_tar
from artemis.utils.sql_utils import ArtemisDB, ArtemisSIG
from artemis.utils.update_utils import UpdateManager
from artemis.utils.path_utils import normalize_dialog_path
from artemis.utils.path_utils import DATA_DIR
from artemis.utils.config_utils import CONFIGURE_QT

from artemis.ui.preferences import UIPreferences
from artemis.ui.dbmanager import UIdbmanager
from artemis.ui.signaleditor import UIsignaleditor
from artemis.ui.downloader import UIDownloader
from artemis.ui.spaceweather import UIspaceweather
from artemis.ui.documentsmanager import UIdocumentsmanager
from artemis.ui.categoryeditor import UIcategoryeditor
from artemis.ui.filters import FiltersManager


class UIArtemis(QObject):
    # Python > QML Signals
    close_ui = Signal()
    populate_sig_list = Signal(list)
    populate_sig_details = Signal(list)
    
    clear_list = Signal()
    clear_signal_page = Signal()
    lock_audio_player = Signal()
    lock_menu = Signal(bool, bool)

    show_dialog_popup = Signal(str, str, str)
    show_dialog_download_db = Signal(str, str, str)
    show_dialog_update_artemis = Signal(str, str, str, bool)
    update_info_bar = Signal(str, str)


    def __init__(self):
        super().__init__()

        # Main UI initialization
        self._engine = QQmlApplicationEngine()
        self._engine.rootContext().setContextProperty('APPLICATION_VERSION', Constants.APPLICATION_VERSION)
        self._engine.rootContext().setContextProperty('PYTHON_VERSION', Constants.PYTHON_VERSION)
        self._engine.rootContext().setContextProperty('QT_VERSION', Constants.QT_VERSION)
        self._engine.load('qrc:/ui/Artemis.qml')
        self._window = self._engine.rootObjects()[0]

        self._window_signal = self._window.findChild(QObject, "signalPageObj")
        self._window_signal_list = self._window.findChild(QObject, "signalListObj")

        self.loaded_db = None
        self.loaded_sig = None

        self._connect()

        # Creating istances for other windows
        self.preferences = UIPreferences(self)
        self.dbmanager = UIdbmanager(self)
        self.spaceweather = UIspaceweather(self)
        self.docmanager = UIdocumentsmanager(self)
        self.sigeditor = UIsignaleditor(self)
        self.cateditor = UIcategoryeditor(self)
        self.downloader = UIDownloader(self)

        self.filters_manager = FiltersManager(self)
        self.update_manager = UpdateManager(self)

        self.autoload_db()


    def _connect(self):
        # QML > Python connections
        self._window.showDBmanager.connect(self.show_dbmanager_ui)
        self._window.loadSignal.connect(self.load_sig)
        self._window.showPref.connect(self.show_pref_ui)
        self._window.openSigEditor.connect(self.open_sig_editor)
        self._window.checkForUpdate.connect(self.check_for_update)
        self._window.updateDb.connect(self.update_db)
        self._window.updateArtemis.connect(self.update_artemis)
        self._window.showSpaceWeather.connect(self.show_space_weather_ui)
        self._window.openDbDirectory.connect(self.open_db_directory)
        self._window.showCatManager.connect(self.open_cat_manager)
        
        self._window.newDb.connect(self.new_db)
        self._window.exportDb.connect(self.export_db)
        self._window.importDb.connect(self.import_db)

        self._window_signal.openDocManager.connect(self.show_documentsmanager_ui)
        self._window_signal.openSigEditor.connect(self.open_sig_editor)
        self._window_signal.deleteCatTag.connect(self.delete_cat_tag)
        self._window_signal.addCatTag.connect(self.add_cat_tag)

        # Python > QML connections
        self.close_ui.connect(self._window.close)
        self.update_info_bar.connect(self._window.bottomInfoBar)
        self.show_dialog_popup.connect(self._window.openGeneralDialog)
        self.show_dialog_download_db.connect(self._window.openDialogDownloadDb)
        self.show_dialog_update_artemis.connect(self._window.openDialogUpdateArtemis)
        self.lock_menu.connect(self._window.lockMenu)

        self.populate_sig_list.connect(self._window_signal_list.populateList)
        self.clear_list.connect(self._window_signal_list.clearList)

        self.populate_sig_details.connect(self._window_signal.populateSignalParam)
        self.lock_audio_player.connect(self._window_signal.lockPlayer)
        self.clear_signal_page.connect(self._window_signal.resetAll)


    def load_db(self, db_dir_name):
        """ Load the DB and populate the signals list

        Args:
            db_dir_name (str): folder name in the data folder
        """
        # Loading DB
        self.loaded_db = ArtemisDB(db_dir_name)
        self.loaded_db.load()
        # Clearing UI
        self.lock_menu.emit(False, self.loaded_db.is_sigid)
        self.clear_signal_page.emit()
        # Populating UI
        self.filters_manager.load_filter_lists()
        self.populate_sig_list.emit(self.loaded_db.all_signals)
        # Updating status bar
        self.bottom_info_bar(f"{self.loaded_db.name} v{self.loaded_db.version} | {self.loaded_db.count_signals} signals", "info")


    @Slot(int)
    def load_sig(self, sig_id):
        """ Load the selected signal and populate the SignalPage

        Args:
            sig_id (int): SIG_ID of the signal to be loaded 
        """
        self.loaded_sig = ArtemisSIG(self.loaded_db)
        self.loaded_sig.load(sig_id)

        self.populate_sig_details.emit([self.loaded_sig.summary])


    def show_pref_ui(self):
        """ Load the preference windows
        """
        self.preferences.load_preferences_ui()


    def show_dbmanager_ui(self):
        """ Load the DB manager windows
        """
        self.dbmanager.load_dbmanager_ui()


    @Slot(str, list, bool)
    def open_sig_editor(self, type, sig_param, is_new):
        """ Open the signal editor windows
            Called when the user want to add, edit or delete the signal or its parametes.
        """
        self.sigeditor.load_signaleditor_ui(type, sig_param, is_new)


    def show_space_weather_ui(self):
        """ Open the space weather windows
        """
        self.spaceweather.load_spaceweather_ui()


    def show_documentsmanager_ui(self):
        """ Open the documents manager windows
        """
        self.docmanager.load_documentsmanager_ui()


    def check_for_update(self):
        """ User manual check for updates updates
        """
        self.update_manager.update(show_popup=True)


    def set_update_available(self, available):
        """ Set the blinking light when updates are available
        """
        self._window.setProperty("updateAvailable", available)


    def dialog_download_db(self, message_type, title, message):
        """ Dialog popup for DB download confirmation
        """
        self.show_dialog_download_db.emit(message_type, title, message)


    def dialog_update_artemis(self, message_type, title, message, auto=False):
        """ Dialog popup for Artemis download confirmation
        """
        self.show_dialog_update_artemis.emit(message_type, title, message, auto)


    @Slot()
    def update_db(self):
        """ Start the download of the sigID DB
        """
        self.update_manager.download_db()


    @Slot()
    def update_artemis(self):
        """ Start the download of Artemis
        """
        self.update_manager.download_artemis()


    def open_db_directory(self):
        """ Open the local folder of the loaded DB
        """
        open_directory(self.loaded_db.db_dir)


    @Slot(str)
    def new_db(self, name):
        """ Create a new local DB

        Args:
            name (str): name of the new DB, hardcoded in sql info table
        """
        try:
            new_db = ArtemisDB(str(uuid.uuid4()))
            new_db.create(name)
            self.load_db(new_db.db_dir_name)
            self.dialog_popup(
                Messages.DIALOG_TYPE_INFO,
                Messages.GENERIC_SUCCESS,
                Messages.DB_CREATION_SUCCESS_MSG
            )
        except Exception as e:
            self.dialog_popup(
                Messages.DIALOG_TYPE_ERROR,
                Messages.GENERIC_ERROR,
                Messages.GENERIC_ERROR_MSG.format(e)
            )


    @Slot(str)
    def export_db(self, save_path):
        """ Export the load DB in a tar file. Does not use compression

        Args:
            save_path (str): destination path of the generated .tar file
        """
        try:
            url = QUrl(save_path)
            dest_path = url.toLocalFile()

            make_tar(dest_path, self.loaded_db.db_dir)
            self.dialog_popup(
                Messages.DIALOG_TYPE_INFO,
                Messages.GENERIC_SUCCESS,
                Messages.EXPORTING_SUCCESS_MSG
            )
        except Exception as e:
            self.dialog_popup(
                Messages.DIALOG_TYPE_ERROR,
                Messages.GENERIC_ERROR,
                Messages.GENERIC_ERROR_MSG.format(e)
            )


    @Slot(str)
    def import_db(self, tar_path):
        """ Import a new DB in the Artemis data folder

        Args:
            tar_path (str): Path of the archive to be imported
        """
        try:
            origin_path = normalize_dialog_path(tar_path)
            save_path = DATA_DIR / str(uuid.uuid4())
            unpack_tar(origin_path, save_path)
            self.dialog_popup(
                Messages.DIALOG_TYPE_INFO,
                Messages.GENERIC_SUCCESS,
                Messages.IMPORTING_SUCCESS_MSG
            )
        except Exception as e:
            self.dialog_popup(
                Messages.DIALOG_TYPE_ERROR,
                Messages.GENERIC_ERROR,
                Messages.GENERIC_ERROR_MSG.format(e)
            )


    @Slot(int)
    def add_cat_tag(self, clb_id):
        self.loaded_sig.insert_category(clb_id)
        self.load_db(self.loaded_db.db_dir_name)


    @Slot(int)
    def delete_cat_tag(self, cat_id):
        self.loaded_sig.delete_category(cat_id)
        self.load_db(self.loaded_db.db_dir_name)


    def open_cat_manager(self):
        """ Open the category manager windows
        """
        self.cateditor.load_cateditor_ui()


    def autoload_db(self):
        """ This will autoload the latest local sigID DB, if present
            according to the user settings
        """
        sig_id_db = self.dbmanager.get_latest_local_sigid_db()
        autoload = CONFIGURE_QT.value("Database", "autoload", 0)
        if sig_id_db is not None and int(autoload):
            self.load_db(sig_id_db.db_dir_name)


    def dialog_popup(self, message_type, title, message):
        """ Opens a general dialog popup

        Args:
            message_type (str): 'info', 'question', 'warn', 'error'
            title (str): header of the dialoog
            message (sstr): description inside the dialog
        """
        self.show_dialog_popup.emit(message_type, title, message)


    @Slot(str, str)
    def bottom_info_bar(self, message, message_type):
        """ Manage the footer info bar

        Args:
            message (str): text to be shown in the info bar
            message_type (str): 'info', 'warning'
        """
        self.update_info_bar.emit(message, message_type)



from datetime import datetime


def format_frequency(freq_hz):
    """ Return frequency in a human-readable format

    Args:
        freq_hz (int): frequency in Hz
    """
    scale = _change_unit_freq(freq_hz)
    formatted_freq = f'{freq_hz / scale[0]} {scale[1]}'
    return formatted_freq


def _change_unit_freq(freq_hz):
    """ Return a scale factor and unit based on the number of digits in the frequency

    Args:
        freq_hz (int): frequency in Hz
    """
    digits = len(str(freq_hz))

    if digits < 4:
        return 1, 'Hz'
    elif digits < 7:
        return 10**3, 'kHz'
    elif digits < 10:
        return 10**6, 'MHz'
    else:
        return 10**9, 'GHz'


def parse_date(date_str):
    """ Parses a date string in "%Y-%m-%d %H:%M:%S.%f" format and returns
    the date in "YYYY-MM-DD" format. If parsing fails, returns the original string.

    Args:
        date_str (str): The date string to parse.
    """
    try:
        form_date = datetime.strptime(date_str, "%Y-%m-%d %H:%M:%S.%f")
        return str(form_date.date())
    except ValueError:
        return date_str
